---------------------------------------------------------------------------
-- DEMO : Recherche en texte intgral
-- Base Wikipedia FR
-- Dmo DATAFROGS 2023
-- https://blog.datafly.pro
---------------------------------------------------------------------------

USE Wikipedia
GO

-- Table des articles de wikipedia
--CREATE TABLE Articles
--(ID INT NOT NULL IDENTITY,
--Titre VARCHAR(255),
--Article VARCHAR(MAX)
--CONSTRAINT PK_Article PRIMARY KEY CLUSTERED (ID))
--TEXTIMAGE_ON LOB

SELECT COUNT(*) FROM Articles
SELECT * FROM Articles TABLESAMPLE (100 ROWS)

SELECT * FROM Articles WHERE Article LIKE '% Agatha Christie %'  --1mn30s

---------------------------------------------------------------------------
-- Configuration
---------------------------------------------------------------------------
-- Cration du catalogue
CREATE FULLTEXT CATALOG FTC_Wikipedia
   ON FILEGROUP FG_Wikipedia_FTC
     WITH ACCENT_SENSITIVITY = OFF
   AS DEFAULT;
GO
-- Cration de l'index Full-Text
CREATE FULLTEXT INDEX
   ON dbo.Articles (Titre LANGUAGE 1036, Article LANGUAGE 1036) 
   KEY INDEX PK_Article
       ON (FTC_Wikipedia, FILEGROUP FG_Wikipedia_FTC)
   WITH  (CHANGE_TRACKING = AUTO,   -- MANUAL|AUTO|  OFF[,NO POPULATION]
          STOPLIST        = SYSTEM);

-- Stoplist personnalise possible
CREATE FULLTEXT STOPLIST MyStopList
ALTER FULLTEXT STOPLIST MyStopList ADD 'MonMotNoir' LANGUAGE French;

-- Ou  partir d'une liste existante
CREATE FULLTEXT STOPLIST MyStopList
FROM SYSTEM STOPLIST -- ou Mabase.maStopList

-- Informations sur le catalogue
SELECT
cat.name AS [Name],
cat.fulltext_catalog_id AS [ID],
CAST(FULLTEXTCATALOGPROPERTY(cat.name,'AccentSensitivity') AS BIT) AS [IsAccentSensitive],
CAST(cat.is_default AS BIT) AS [IsDefault],
dp.name AS [Owner],
FULLTEXTCATALOGPROPERTY(cat.name,'LogSize') AS [ErrorLogSize],  -- dprci
FULLTEXTCATALOGPROPERTY(cat.name,'IndexSize') AS [FullTextIndexSize (MB)],
FULLTEXTCATALOGPROPERTY(cat.name,'ItemCount') AS [ItemCount]
,(SELECT CASE FULLTEXTCATALOGPROPERTY(cat.name,'PopulateStatus')
        WHEN 0 THEN 'Idle'
        WHEN 1 THEN 'Full Population In Progress'
        WHEN 2 THEN 'Paused'
        WHEN 3 THEN 'Throttled'
        WHEN 4 THEN 'Recovering'
        WHEN 5 THEN 'Shutdown'
        WHEN 6 THEN 'Incremental Population In Progress'
        WHEN 7 THEN 'Building Index'
        WHEN 8 THEN 'Disk Full.  Paused'
        WHEN 9 THEN 'Change Tracking' END) AS PopulateStatus
FROM
sys.fulltext_catalogs AS cat
LEFT OUTER JOIN sys.filegroups AS fg ON cat.data_space_id = fg.data_space_id
LEFT OUTER JOIN sys.database_principals AS dp ON cat.principal_id=dp.principal_id

-- Taille Donnes/Catalogue ?
SELECT OBJECT_NAME(i.object_id) AS TableName, i.type_desc AS IndexType, SUM(a.total_pages * 8 / 1024) AS Size_Mo
FROM sys.tables t
     JOIN sys.indexes i ON t.object_id=i.object_id
     JOIN sys.partitions p ON i.object_id=p.object_id AND i.index_id=p.index_id
     JOIN sys.allocation_units a ON p.partition_id=a.container_id
WHERE t.type='U' AND i.index_id=1
GROUP BY OBJECT_NAME(i.object_id), i.type_desc
UNION
SELECT 'Articles', 'FullText', FULLTEXTCATALOGPROPERTY('FTC_Wikipedia', 'IndexSize');

-- Surveiller la population de l'index
SELECT * FROM sys.dm_fts_index_population WHERE database_id = DB_ID()

---------------------------------------------------------------------------
-- Queries
---------------------------------------------------------------------------

---------------------------------------------------------------------------
-- prdicat CONTAINS
---------------------------------------------------------------------------
SET STATISTICS IO,TIME ON
GO
-- Recherche d'un mot
SELECT TOP 1000 *
FROM Articles A
WHERE CONTAINS(*,'"anticonstitutionnellement"')
ORDER BY A.Titre

-- mot qui commence par
SELECT TOP 1000 *
FROM Articles A
WHERE CONTAINS(*,'"anticonstitutionnel*"')
ORDER BY A.Titre

-- Expression
SELECT TOP 1000 *
FROM Articles
WHERE CONTAINS(Titre,'"Agatha Christie"')
ORDER BY Titre

-- Plusieurs expressions
SELECT TOP 1000 *
FROM Articles
WHERE CONTAINS((titre,article),'"Hercule Poirot" AND "Crime de l''Orient Express"')
ORDER BY Titre

SELECT TOP 1000 *
FROM Articles
WHERE CONTAINS((titre,article),'"Hercule Poirot" AND "Crime de l''Orient Express" AND NOT "Agatha Christie"')
ORDER BY Titre

-- Plusieurs expressions : proximit
SELECT TOP 1000 *
FROM Articles
WHERE CONTAINS((article),'"Hercule Poirot" NEAR "Miss Marple"')
ORDER BY Titre

SELECT TOP 1000 *
FROM Articles
WHERE CONTAINS((article),'"Hercule Poirot" AND "Miss Marple"')
ORDER BY Titre

SELECT TOP 1000 *
FROM Articles
WHERE CONTAINS((article),'NEAR(("Hercule Poirot", "Miss Marple"))')
ORDER BY Titre

-- Plusieurs expressions : proximit, distance de 2 mots max, dans l'ordre
SELECT TOP 1000 *
FROM Articles
WHERE CONTAINS((article),'NEAR(("Hercule Poirot", "Miss Marple") ,2,TRUE)')
ORDER BY Titre

SELECT TOP 1000 *
FROM Articles
WHERE CONTAINS((article),'NEAR(("Hercule Poirot", "Miss Marple") ,2,FALSE)')
ORDER BY Titre

SELECT TOP 1000 *
FROM Articles
WHERE CONTAINS((article),'NEAR(("Hercule Poirot", "Miss Marple") ,0,TRUE)')
ORDER BY Titre

-- Formes flchies
SELECT *
FROM Articles
WHERE  CONTAINS(titre, 'FORMSOF (INFLECTIONAL, "tuer")', LANGUAGE 'french' )

---------------------------------------------------------------------------
-- Thsaurus et parser
-- Expansions et replacements
---------------------------------------------------------------------------

/* Modification du fichier de thsaurus franais
<XML ID="Microsoft Search Thesaurus">
    <thesaurus xmlns="x-schema:tsSchema.xml">
	<diacritics_sensitive>0</diacritics_sensitive>
        <expansion>
            <sub>Crime</sub>
            <sub>Homicide</sub>
            <sub>Meurtre</sub>
            <sub>Assassinat</sub>
        </expansion>
    </thesaurus>
</XML>
*/
-- Charger le thsaurus
EXEC sys.sp_fulltext_load_thesaurus_file 1036;

SELECT *
FROM Articles
WHERE  CONTAINS(titre, 'FORMSOF (THESAURUS , "meurtre") AND "Agatha Christie"', LANGUAGE 'french')

-- Fonctionnement du parsing : analyseur lexical, synonymes, liste de mots vides
SELECT * FROM sys.dm_fts_parser('FORMSOF (THESAURUS , "meurtre")',1036,0,1)

SELECT * FROM sys.dm_fts_parser
('"Agatha Christie est une femme de lettres britannique, auteur de nombreux romans policiers, ne le 15 septembre 1890  Torquay, Royaume-Uni, et morte le 12 janvier 1976  Wallingford au Royaume-Uni."',1036,0,1)

---------------------------------------------------------------------------
-- Prdicat FREETEXT : mots distincts, formes flchies, thsaurus
---------------------------------------------------------------------------

SELECT *
FROM Articles
WHERE  CONTAINS(Titre, '"Meurtre au soleil"');

SELECT *
FROM Articles
WHERE  FREETEXT(Titre, '"Meurtre au soleil"');


-------------------------------------------------
-- Fonction CONTAINSTABLE : RANK
-------------------------------------------------

SELECT * FROM CONTAINSTABLE (Articles, *, 
                                 'FORMSOF (INFLECTIONAL, "meurtre")
                                   AND "Hercule Poirot" 
								   AND "Roman policier"
								   AND "Agatha Christie"')

SELECT *
FROM   Articles A
       INNER JOIN CONTAINSTABLE (Articles, Article, 
                                 'FORMSOF (INFLECTIONAL, "meurtre")
                                    AND "Hercule Poirot" 
								    AND "Roman policier"
								    AND "Agatha Christie"') AS CT
         ON A.ID = CT.[KEY]
ORDER BY RANK DESC        


-- et la possibilit de limit le rang  n valeurs :
SELECT *
FROM   Articles A
       INNER JOIN CONTAINSTABLE (Articles, *, 
                                 'FORMSOF (INFLECTIONAL, "meurtre")
                                    AND "Hercule Poirot" 
								    AND "Roman policier"
								    AND "Agatha Christie"',10) AS CT
         ON A.ID = CT.[KEY]
     

-- donner des poids aux mots :
SELECT *
FROM   Articles A
       INNER JOIN CONTAINSTABLE (Articles, *, 
                                 'ISABOUT ( FORMSOF (INFLECTIONAL, "meurtre") weight (.8),
                                   "Hercule Poirot" weight (.2),
								   "Roman policier" weight (.5),
								   "Agatha Christie" weight (.9))') AS CT
         ON A.ID = CT.[KEY]
ORDER BY RANK DESC    

-------------------------------------------------
-- Fonction FREETEXTTABLE : RANK
-------------------------------------------------

SELECT * FROM FREETEXTTABLE (dbo.Articles, Titre, 
                                 'Meurtre au soleil')

SELECT *
FROM   Articles AS A
       INNER JOIN FREETEXTTABLE (dbo.Articles, Titre, 
                                 'Meurtre au soleil') AS CT
         ON A.ID = CT.[KEY]
ORDER BY RANK DESC

SELECT *
FROM   Articles AS A
       INNER JOIN FREETEXTTABLE (dbo.Articles, Titre, 
                                 'Meurtre au soleil',10) AS CT
         ON A.ID = CT.[KEY]

